﻿Imports System.IO.Ports

Public Class Linefollower
    'everything needed for:
    'Bluetooth communication these are public so that they can be changed in a different form
    Public comPortOut As String = "COM8"
    Public Baudrate As String = "115200"
    Public BluethOut As New SerialPort

    'To the path files these are public so that they can be changed in a different form
    Public SaveDirectionHistory As String = "C:\Program Files\Linefollower\Values\Values.txt"
    Public SaveDirectionSettings As String = "C:\Program Files\Linefollower\Settings\Settings.txt"

    'Saving data
    Public speed As Single = 0.0
    Public average As String = "No"
    Public laps As Integer = 1
    Public time As String = "00:00:000"
    Public track As String = "4144"

    'Status check
    Dim timesTicked As Integer
    Dim timePastSec As Integer = 0
    Dim timePastMin As Integer = 0

    'Configuring serial settings
    Public Sub configureSerial()

        'settings for Bluetooth
        BluethOut.BaudRate = Baudrate
        BluethOut.DataBits = 8
        BluethOut.StopBits = StopBits.One
        BluethOut.Handshake = Handshake.None
        BluethOut.Parity = Parity.None
        BluethOut.ReadTimeout = 1000

        BluethOut.DtrEnable = True
        BluethOut.RtsEnable = True

        BluethOut.PortName = comPortOut

    End Sub

    'What needs to happen when you connect with Bluetooth
    Public Sub connection()

        'Writing to the Serial monitor (of the vb program)
        Serial_monitor.tbxSerialMonitor.AppendText(vbCrLf & "Trying to open Bluetooth Serial" & vbCrLf)

        nudKp.Enabled = True
        nudKi.Enabled = True
        nudKd.Enabled = True
        nudSpeed.Enabled = True
        nudCycleTime.Enabled = True

        btnPID.Enabled = True
        btnSpeed.Enabled = True
        btnCycletime.Enabled = True
        btnEEPROM.Enabled = True
        btnStart.Enabled = True
        btnStop.Enabled = True
        btnCheckStatus.Enabled = True

        pbxNoBluetooth.Visible = False
        pbxBluetooth.Visible = True

        tsmiManualControlSystem.Enabled = True
        tsmiCalibrate.Enabled = True

    End Sub

    'What needs to happen when you disconnect with Bluetooth
    Public Sub connectionLost()

        Serial_monitor.tbxSerialMonitor.AppendText(vbCrLf & "Closing Bluetooth Serial" & vbCrLf)

        nudKp.Enabled = False
        nudKi.Enabled = False
        nudKd.Enabled = False
        nudSpeed.Enabled = False
        nudCycleTime.Enabled = False

        btnPID.Enabled = False
        btnSpeed.Enabled = False
        btnCycletime.Enabled = False
        btnEEPROM.Enabled = False
        btnStart.Enabled = False
        btnStop.Enabled = False
        btnCheckStatus.Enabled = False

        pbxNoBluetooth.Visible = True
        pbxBluetooth.Visible = False

        Serial_monitor.tbxSerialMonitor.AppendText(vbCrLf & "Bluetooth Serial closed" & vbCrLf)

        tsmiConnect.Enabled = True
        tsmiDisconnect.Enabled = False


        tsmiManualControlSystem.Enabled = False
        tsmiCalibrate.Enabled = False

    End Sub

    'If you press connect in the toolstrip
    Private Sub tsmiConnect_Click(sender As Object, e As EventArgs) Handles tsmiConnect.Click

        connection()

        Try

            configureSerial()

            BluethOut.Open()

            Serial_monitor.tbxSerialMonitor.AppendText("Bluetooth Serial Opened" & vbCrLf)

            tsmiConnect.Enabled = False
            tsmiDisconnect.Enabled = True


        Catch ex As Exception

            Dim result As DialogResult

            result = MessageBox.Show("Connection lost." & vbCrLf & "Ceck if bluetooth device is on." & vbCrLf & "Try to reconnect?", "ERROR", MessageBoxButtons.YesNo, MessageBoxIcon.Error)

            If (result = DialogResult.No) Then

                connectionLost()

            Else

                connectionLost()
                tsmiConnect.PerformClick()

            End If

        End Try

    End Sub

    Private Sub tsmiDisconnect_Click(sender As Object, e As EventArgs) Handles tsmiDisconnect.Click

        Try

            BluethOut.Close()

            connectionLost()

        Catch ex As Exception

            Dim result As DialogResult

            result = MessageBox.Show("Already disconnected." & vbCrLf & vbCrLf & "Reconnect?", "ERROR", MessageBoxButtons.YesNo, MessageBoxIcon.Error)

            If (result = DialogResult.No) Then

                connectionLost()

            Else

                connectionLost()
                tsmiConnect.PerformClick()

            End If

        End Try

    End Sub

    Private Sub tsmiManualControlSystem_Click(sender As Object, e As EventArgs) Handles tsmiManualControlSystem.Click

        If (Manual_Control_System.Visible = True) Then

            Manual_Control_System.Focus()

        Else

            Manual_Control_System.Show()

        End If

    End Sub

    Private Sub tsmiChange_Click(sender As Object, e As EventArgs) Handles tsmiChange.Click

        If (BluethOut.IsOpen = True) Then

            Dim result As DialogResult

            result = MessageBox.Show("Bluetooth needs to be disconnected first." & vbCrLf & "Do you want to proceed?", "Disconnecting", MessageBoxButtons.YesNo, MessageBoxIcon.Information)

            If (result = DialogResult.Yes) Then

                tsmiDisconnect.PerformClick()

                Bluetooth_Settings.Show()

            End If

        Else

            Bluetooth_Settings.Show()

        End If

    End Sub

    Private Sub tsmiShow_Click(sender As Object, e As EventArgs) Handles tsmiShow.Click

        Dim result As DialogResult

        result = MessageBox.Show("The COM port is: " & comPortOut & vbCrLf & "The baudrate is: " & Baudrate & vbCrLf & vbCrLf & "Do you want to change these?", "Settings Bluetooth", MessageBoxButtons.YesNo, MessageBoxIcon.Information)

        If (result = DialogResult.Yes) Then

            tsmiChange.PerformClick()

        End If

    End Sub

    Private Sub tsmiSaveSettings_Click(sender As Object, e As EventArgs) Handles tsmiSaveSettings.Click

        Try

            Dim saveSettings As String

            saveSettings = (nudKp.Value & vbCrLf & nudKi.Value & vbCrLf & nudKd.Value & vbCrLf & nudSpeed.Value & vbCrLf & nudCycleTime.Value & vbCrLf & comPortOut & vbCrLf & Baudrate)

            My.Computer.FileSystem.WriteAllText(SaveDirectionSettings, saveSettings, False)

            MessageBox.Show("Settings are saved", "Saved", MessageBoxButtons.OK, MessageBoxIcon.Asterisk)

        Catch ex As Exception

            MessageBox.Show("An Error occurred while saving the settings." & vbCrLf & "Try again later.", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error)

        End Try


    End Sub

    Private Sub tsmiLoadSettings_Click(sender As Object, e As EventArgs) Handles tsmiLoadSettings.Click

        Try

            Dim savedSettings() As String

            savedSettings = Split(My.Computer.FileSystem.ReadAllText(SaveDirectionSettings), vbCrLf)

            nudKp.Value = savedSettings(0)
            nudKi.Value = savedSettings(1)
            nudKd.Value = savedSettings(2)
            nudSpeed.Value = savedSettings(3)
            nudCycleTime.Value = savedSettings(4)
            comPortOut = savedSettings(5)
            Baudrate = savedSettings(6)

        Catch ex As Exception

            MessageBox.Show("An Error occurred while loading the settings." & vbCrLf & "Try again later.", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error)

        End Try

    End Sub

    Private Sub tsmiChangeHistoryFilePath_Click(sender As Object, e As EventArgs) Handles tsmiChangeHistoryFilePath.Click

        If (ChangeHistoryFilePath.Visible = True) Then

            ChangeHistoryFilePath.Focus()

        Else

            ChangeHistoryFilePath.Show()

        End If

    End Sub

    Private Sub tsmiCalibrate_Click(sender As Object, e As EventArgs) Handles tsmiCalibrate.Click

        Try

            Dim result As DialogResult

            result = MessageBox.Show("Place the sensors on a white background.", "Calibrating White", MessageBoxButtons.OKCancel, MessageBoxIcon.Information)

            If (result = DialogResult.OK) Then

                BluethOut.WriteLine("Calibrate white" & vbCrLf)

                result = MessageBox.Show("White calibrated." & vbCrLf & "Place the sensors on a black background.", "Calibrating black", MessageBoxButtons.OKCancel, MessageBoxIcon.Information)

                If (result = DialogResult.OK) Then

                    BluethOut.WriteLine("Calibrate black" & vbCrLf)
                    MessageBox.Show("Black calibrated.", "Calibrated", MessageBoxButtons.OK, MessageBoxIcon.Information)

                End If

            End If

        Catch ex As Exception

            Dim result As DialogResult

            result = MessageBox.Show("Connection lost." & vbCrLf & "Ceck if bluetooth device is on." & vbCrLf & "Try to reconnect?", "ERROR", MessageBoxButtons.YesNo, MessageBoxIcon.Error)

            If (result = DialogResult.No) Then

                connectionLost()

            Else

                connectionLost()
                tsmiConnect.PerformClick()

            End If

        End Try

    End Sub

    Private Sub tsmiSerialMonitor_Click(sender As Object, e As EventArgs) Handles tsmiSerialMonitor.Click

        If (Serial_monitor.Visible = True) Then

            Serial_monitor.Focus()

        Else

            Serial_monitor.Show()

        End If

    End Sub

    Private Sub btnLoadHistory_Click(sender As Object, e As EventArgs) Handles btnLoadHistory.Click

        Try

            Dim Values As String

            Values = My.Computer.FileSystem.ReadAllText(SaveDirectionHistory)
            tbxHistory.Text = Values
            tbxHistory.AppendText(vbCrLf)
            tbxHistory.ScrollToCaret()

        Catch ex As Exception

            MessageBox.Show("An Error occurred while loading the history values." & vbCrLf & "Try again later.", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error)

        End Try

    End Sub

    Private Sub btnSaveHistory_Click(sender As Object, e As EventArgs) Handles btnSaveHistory.Click

        Try

            If (gbxRemarks.Visible = False) Then

                gbxRemarks.Visible = True
                tbxRemarks.Visible = True
                gbxStatus.Visible = False

            Else

                'save values in this order:
                'kp (tab) ki (tab) kd (tab) speed (2tab) average (2tab) laps (tab) track (2tab) PWM (tab) cycletime (2tab) time (vbclrf)
                'remarks:
                Dim savetext As String
                Dim kp As String = nudKp.Value.ToString
                Dim ki As String = nudKi.Value.ToString
                Dim kd As String = nudKd.Value.ToString
                Dim speedMpS As String = speed.ToString
                Dim speedPWM As String = nudSpeed.Value.ToString
                Dim cycletime As String = nudCycleTime.Value.ToString
                Dim Remarks As String = tbxRemarks.Text
                Dim Line As String = "******************************************************************************************************************************************************************************************"

                savetext = (Line & vbCrLf & kp & vbTab & ki & vbTab & kd & vbTab & speedMpS & vbTab & vbTab & average & vbTab & vbTab & laps & vbTab & track & vbTab & vbTab & speedPWM & vbTab & cycletime & vbTab & vbTab & time & vbCrLf & "Remarks: " & Remarks & vbCrLf & Line & vbCrLf)

                My.Computer.FileSystem.WriteAllText(SaveDirectionHistory, savetext, True)

                tbxRemarks.Clear()

                gbxRemarks.Visible = False
                tbxRemarks.Visible = False
                gbxStatus.Visible = True

            End If

        Catch ex As Exception

            MessageBox.Show("An Error occurred while saving the history values." & vbCrLf & "Try again later.", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error)

        End Try

    End Sub

    Private Sub btnPID_Click(sender As Object, e As EventArgs) Handles btnPID.Click

        Try

            Dim pterm As String = nudKp.Value
            Dim iterm As String = nudKi.Value
            Dim dterm As String = nudKd.Value

            pterm = pterm.Replace(",", ".")
            iterm = iterm.Replace(",", ".")
            dterm = dterm.Replace(",", ".")


            BluethOut.WriteLine("PID " & pterm & " " & iterm & " " & dterm & vbCrLf)

        Catch ex As Exception

            Dim result As DialogResult

            result = MessageBox.Show("Connection lost." & vbCrLf & "Ceck if bluetooth device is on." & vbCrLf & "Try to reconnect?", "ERROR", MessageBoxButtons.YesNo, MessageBoxIcon.Error)

            If (result = DialogResult.No) Then

                connectionLost()

            Else

                connectionLost()
                tsmiConnect.PerformClick()

            End If

        End Try

    End Sub

    Private Sub btnSpeed_Click(sender As Object, e As EventArgs) Handles btnSpeed.Click

        Try

            BluethOut.WriteLine("set speed " & nudSpeed.Value & vbCrLf)

        Catch ex As Exception

            Dim result As DialogResult

            result = MessageBox.Show("Connection lost." & vbCrLf & "Ceck if bluetooth device is on." & vbCrLf & "Try to reconnect?", "ERROR", MessageBoxButtons.YesNo, MessageBoxIcon.Error)

            If (result = DialogResult.No) Then

                connectionLost()

            Else

                connectionLost()
                tsmiConnect.PerformClick()

            End If

        End Try

    End Sub

    Private Sub btnCycletime_Click(sender As Object, e As EventArgs) Handles btnCycletime.Click

        Try

            BluethOut.WriteLine("set cycleTime " & nudCycleTime.Value & vbCrLf)

        Catch ex As Exception

            Dim result As DialogResult

            result = MessageBox.Show("Connection lost." & vbCrLf & "Ceck if bluetooth device is on." & vbCrLf & "Try to reconnect?", "ERROR", MessageBoxButtons.YesNo, MessageBoxIcon.Error)

            If (result = DialogResult.No) Then

                connectionLost()

            Else

                connectionLost()
                tsmiConnect.PerformClick()

            End If

        End Try

    End Sub

    Private Sub btnEEPROM_Click(sender As Object, e As EventArgs) Handles btnEEPROM.Click

        Try

            BluethOut.WriteLine("WriteToEEPROM" & vbCrLf)

        Catch ex As Exception

            Dim result As DialogResult

            result = MessageBox.Show("Connection lost." & vbCrLf & "Ceck if bluetooth device is on." & vbCrLf & "Try to reconnect?", "ERROR", MessageBoxButtons.YesNo, MessageBoxIcon.Error)

            If (result = DialogResult.No) Then

                connectionLost()

            Else

                connectionLost()
                tsmiConnect.PerformClick()

            End If

        End Try

    End Sub

    Private Sub btnStart_Click(sender As Object, e As EventArgs) Handles btnStart.Click

        Try

            BluethOut.WriteLine("start" & vbCrLf)

            pbxRunning.BackColor = Color.Lime

        Catch ex As Exception

            Dim result As DialogResult

            result = MessageBox.Show("Connection lost." & vbCrLf & "Ceck if bluetooth device is on." & vbCrLf & "Try to reconnect?", "ERROR", MessageBoxButtons.YesNo, MessageBoxIcon.Error)

            If (result = DialogResult.No) Then

                connectionLost()

            Else

                connectionLost()
                tsmiConnect.PerformClick()

            End If

        End Try

    End Sub

    Private Sub btnStop_Click(sender As Object, e As EventArgs) Handles btnStop.Click

        Try

            BluethOut.WriteLine("stop" & vbCrLf)

            pbxRunning.BackColor = Color.Red

        Catch ex As Exception

            Dim result As DialogResult

            result = MessageBox.Show("Connection lost." & vbCrLf & "Ceck if bluetooth device is on." & vbCrLf & "Try to reconnect?", "ERROR", MessageBoxButtons.YesNo, MessageBoxIcon.Error)

            If (result = DialogResult.No) Then

                connectionLost()

            Else

                connectionLost()
                tsmiConnect.PerformClick()

            End If

        End Try

    End Sub

    Private Sub tsmiSpeedCheck_Click(sender As Object, e As EventArgs) Handles tsmiSpeedCheck.Click

        If (Measure_speed.Visible = True) Then

            Measure_speed.Focus()

        Else

            Measure_speed.Show()

        End If

    End Sub

    Private Sub btnCheckStatus_Click(sender As Object, e As EventArgs) Handles btnCheckStatus.Click

        Try

            BluethOut.ReadExisting()

            BluethOut.WriteLine("Status" & vbCrLf)
            tmrCheckingStatus.Start()

        Catch ex As Exception

            Dim result As DialogResult

            result = MessageBox.Show("Connection lost." & vbCrLf & "Ceck if bluetooth device is on." & vbCrLf & "Try to reconnect?", "ERROR", MessageBoxButtons.YesNo, MessageBoxIcon.Error)

            If (result = DialogResult.No) Then

                connectionLost()

            Else

                connectionLost()
                tsmiConnect.PerformClick()

            End If

        End Try

    End Sub

    Private Sub tmrCheckingStatus_Tick(sender As Object, e As EventArgs) Handles tmrCheckingStatus.Tick

        Try

            Dim StatusCheck() As String
            Dim calEff As Double

            timesTicked += 1

            tbxStatus.AppendText(BluethOut.ReadExisting)

            If (timesTicked > 1) Then

                StatusCheck = Split(tbxStatus.Text, vbCrLf)

                tmrTimePast.Stop()

                lblTimePast.Text = "Time since last status check: 00:00"
                timePastSec = 0
                timePastMin = 0

                If (IsNumeric(StatusCheck(2))) Then

                    Dim PWMEff As Double

                    PWMEff = (StatusCheck(2) / 255) * 100

                    PWMEff = Math.Round(PWMEff, 2, MidpointRounding.AwayFromZero)

                    lblProPWM.Text = "PWM: " & PWMEff.ToString & "%"

                End If

                If (IsNumeric(StatusCheck(4)) And IsNumeric(StatusCheck(5))) Then

                    calEff = (StatusCheck(5) / StatusCheck(4)) * 100

                    calEff = Math.Round(calEff, 2, MidpointRounding.AwayFromZero)

                    lblEfficiency.Text = "Efficiency: " & calEff.ToString & "%"

                    tmrCheckingStatus.Stop()

                    tmrTimePast.Start()

                    lblCalcCycle.Text = ("Calc: " & StatusCheck(5))
                    lblSetCycle.Text = ("Set: " & StatusCheck(4))

                End If

                Me.Text = StatusCheck(3)

                tbxStatus.Clear()

                If (StatusCheck(1) = 0) Then

                    pbxRunning.BackColor = Color.Red

                ElseIf (StatusCheck(1) = 1) Then

                    pbxRunning.BackColor = Color.Lime

                End If

            End If

        Catch ex As Exception

            lblEfficiency.Text = lblEfficiency.Text & " Not updated!"
            tmrCheckingStatus.Stop()

        End Try

    End Sub

    Private Sub tmrTimePast_Tick(sender As Object, e As EventArgs) Handles tmrTimePast.Tick

        timePastSec += 1

        If (timePastSec = 60) Then

            timePastSec = 0
            timePastMin += 1

        End If

        Dim timesec As String
        Dim timemin As String

        If (timePastSec < 10) Then

            timesec = "0" & timePastSec.ToString

        Else

            timesec = timePastSec.ToString

        End If

        If (timePastMin < 10) Then

            timemin = "0" & timePastMin.ToString

        Else

            timemin = timePastMin.ToString

        End If

        lblTimePast.Text = "Time since last status check: " & timemin & ":" & timesec

    End Sub

    Private Sub Linefollower_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        tsmiLoadSettings.PerformClick()
        Me.KeyPreview = True

    End Sub

    Private Sub Linefollower_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles MyBase.KeyDown

        If (e.Control AndAlso (e.KeyCode = Keys.B)) Then

            If (tsmiConnect.Enabled = True) Then

                tsmiConnect.PerformClick()

            End If

        ElseIf (e.Control AndAlso (e.KeyCode = Keys.D)) Then

            If (tsmiDisconnect.Enabled = True) Then

                tsmiDisconnect.PerformClick()

            End If

        ElseIf (e.Control AndAlso (e.KeyCode = Keys.S)) Then

            tsmiSaveSettings.PerformClick()

        ElseIf (e.Control AndAlso (e.KeyCode = Keys.C)) Then

            If (tsmiCalibrate.Enabled = True) Then

                tsmiCalibrate.PerformClick()

            End If

        ElseIf (e.Alt AndAlso (e.KeyCode = Keys.S)) Then

            tsmiSerialMonitor.PerformClick()

        ElseIf (e.Control AndAlso (e.KeyCode = Keys.V)) Then

            tsmiSpeedCheck.PerformClick()

        End If

    End Sub

End Class
